# wrlifs2vtk copyright (c) 2003 by Charl P. Botha cpbotha@ieee.org
# $Id: wrlifs2vtk.py,v 1.1 2003/08/31 21:54:05 cpbotha Exp $
# convert vrml 97 indexed faced set to VTK ASCII polydata
# I hacked this together for the very simple VRML datasets that are part
# of the previous DSEM visualisation setup... DON'T USE THIS FOR ANYTHING ELSE

# it assumes 3 points per face (i.e. all triangles!)

import re
import sys

def usage():
    print "usage: wrlifs2vtk inputFilename outputFilename"

def lineToScalars(line, nos, cast):
    """Convert text line to nos scalars by using cast.  Throws
    ValueError if that can't happen.
    """
    line.strip()
    splitLine = line.split()
    if len(splitLine) < nos:
        raise ValueError
    else:
        scalars = []
        for e in splitLine[0:nos]:
            scalars.append(cast(e))

        return scalars
    

def main():
    if len(sys.argv) < 3:
        usage()
        return

    inputFilename = sys.argv[1]
    outputFilename = sys.argv[2]

    # search for:
    # geometry IndexedFaceSet {
    #  stuff
    #  coord Coordinate {
    #   point [
    #    float1 float2 float3
    #    ...
    #   ]
    #  }
    #  coordIndex [
    #  index1 index2 index3 -1
    #  ...
    #  ]
    # }

    inputFile = open(inputFilename)
    inputLines = inputFile.readlines()

    inPoint = False
    points = []
    inCI = False
    indices = []
    pointPO = re.compile('\s*point\s*\[')
    ciPO = re.compile('\s*coordIndex\s*\[')
    for line in inputLines:
        if inPoint:
            try:
                floats = lineToScalars(line, 3, float)
                points.append(floats)
            except ValueError:
                inPoint = False
                
        elif inCI:
            try:
                ints = lineToScalars(line, 3, int)
                indices.append(ints)
            except ValueError:
                inCI = False

        if not (inPoint or inCI):
            pointMO = pointPO.match(line)
            if pointMO:
                inPoint = True
            else:
                ciMO = ciPO.match(line)
                if ciMO:
                    inCI = True

    # closes for line...
    # we now have points and indices

    outputFile = open(outputFilename, 'w')
    headerString = \
                 "# vtk DataFile Version 3.0\n" \
                 "generated by wrlifs2vtk.py cpbotha@ieee.org\n" \
                 "ASCII\n" \
                 "DATASET POLYDATA\n"

    outputFile.write(headerString)

    # POINTS numberOfPoints type
    outputFile.write('POINTS %d float\n' % (len(points),))
    for point in points:
        outputFile.write('%.4f %.4f %.4f\n' % (tuple(point)))

    # POLYGONS n size
    # n == number of polygons
    # size == cell list size (number of polygons + all points)
    n = len(indices)
    size = len(indices) + len(indices) * 3
    outputFile.write('POLYGONS %d %d\n' % (n, size))
    for poly in indices:
        outputFile.write('3 %d %d %d\n' % tuple(poly))

    outputFile.write('POINT_DATA %d\n' % (len(points),))
    outputFile.write('CELL_DATA %d\n' % (n,))

    outputFile.close()

if __name__ == "__main__":
    main()
